@@ -1,4 +1,3 @@
-
 module Vectorise.Utils (
   module Vectorise.Utils.Base,
   module Vectorise.Utils.Closure,
@@ -8,6 +7,7 @@ module Vectorise.Utils (
 
   -- * Annotated Exprs
   collectAnnTypeArgs,
+  collectAnnDictArgs,
   collectAnnTypeBinders,
   collectAnnValBinders,
   isAnnTypeArg,
@@ -21,8 +21,8 @@ module Vectorise.Utils (
 
   -- * Naming
   newLocalVar
-) 
-where
+) where
+
 import Vectorise.Utils.Base
 import Vectorise.Utils.Closure
 import Vectorise.Utils.Hoisting
@@ -32,28 +32,41 @@ import Vectorise.Monad
 import Vectorise.Builtins
 import CoreSyn
 import CoreUtils
+import Id
 import Type
 import Control.Monad
 
 
 -- Annotated Exprs ------------------------------------------------------------
+
 collectAnnTypeArgs :: AnnExpr b ann -> (AnnExpr b ann, [Type])
 collectAnnTypeArgs expr = go expr []
   where
     go (_, AnnApp f (_, AnnType ty)) tys = go f (ty : tys)
     go e                             tys = (e, tys)
 
+collectAnnDictArgs :: AnnExpr Var ann -> (AnnExpr Var ann, [AnnExpr Var ann])
+collectAnnDictArgs expr = go expr []
+  where
+    go e@(_, AnnApp f arg) dicts 
+      | isPredTy . exprType . deAnnotate $ arg = go f (arg : dicts)
+      | otherwise                              = (e, dicts)
+    go e                        dicts          = (e, dicts)
+
 collectAnnTypeBinders :: AnnExpr Var ann -> ([Var], AnnExpr Var ann)
 collectAnnTypeBinders expr = go [] expr
   where
-    go bs (_, AnnLam b e) | isTyVar b = go (b:bs) e
+    go bs (_, AnnLam b e) | isTyVar b = go (b : bs) e
     go bs e                           = (reverse bs, e)
 
+-- |Collect all consecutive value binders that are not dictionaries.
+--
 collectAnnValBinders :: AnnExpr Var ann -> ([Var], AnnExpr Var ann)
 collectAnnValBinders expr = go [] expr
   where
-    go bs (_, AnnLam b e) | isId b = go (b:bs) e
-    go bs e                        = (reverse bs, e)
+    go bs (_, AnnLam b e) | isId b 
+                          && (not . isPredTy . idType $ b) = go (b : bs) e
+    go bs e                                                = (reverse bs, e)
 
 isAnnTypeArg :: AnnExpr b ann -> Bool
 isAnnTypeArg (_, AnnType _) = True
@@ -69,58 +82,52 @@ isAnnTypeArg _              = False
 --   in dph-common/D.A.P.Lifted/PArray.hs
 --
 
--- | An empty array of the given type.
-emptyPD :: Type -> VM CoreExpr
-emptyPD = paMethod emptyPDVar "emptyPD"
-
-
--- | Produce an array containing copies of a given element.
-replicatePD
-	:: CoreExpr	-- ^ Number of copies in the resulting array.
-	-> CoreExpr	-- ^ Value to replicate.
-	-> VM CoreExpr
-
-replicatePD len x 
-	= liftM (`mkApps` [len,x])
-        $ paMethod replicatePDVar "replicatePD" (exprType x)
-
-
--- | Select some elements from an array that correspond to a particular tag value
----  and pack them into a new array.
---   eg  packByTagPD Int# [:23, 42, 95, 50, 27, 49:]  3 [:1, 2, 1, 2, 3, 2:] 2 
---          ==> [:42, 50, 49:]
+-- |An empty array of the given type.
 --
-packByTagPD 
-	:: Type		-- ^ Element type.
-	-> CoreExpr	-- ^ Source array.
-	-> CoreExpr	-- ^ Length of resulting array.
-	-> CoreExpr	-- ^ Tag values of elements in source array.
-	-> CoreExpr	-- ^ The tag value for the elements to select.
-	-> VM CoreExpr
+emptyPD :: Type -> VM CoreExpr
+emptyPD = paMethod emptyPDVar emptyPD_PrimVar
 
+-- |Produce an array containing copies of a given element.
+--
+replicatePD :: CoreExpr     -- ^ Number of copies in the resulting array.
+            -> CoreExpr     -- ^ Value to replicate.
+            -> VM CoreExpr
+replicatePD len x 
+  = liftM (`mkApps` [len,x])
+        $ paMethod replicatePDVar replicatePD_PrimVar (exprType x)
+
+-- |Select some elements from an array that correspond to a particular tag value and pack them into a new
+-- array.
+--
+-- > packByTagPD Int# [:23, 42, 95, 50, 27, 49:]  3 [:1, 2, 1, 2, 3, 2:] 2 
+-- >   ==> [:42, 50, 49:]
+--
+packByTagPD :: Type       -- ^ Element type.
+            -> CoreExpr   -- ^ Source array.
+            -> CoreExpr   -- ^ Length of resulting array.
+            -> CoreExpr   -- ^ Tag values of elements in source array.
+            -> CoreExpr   -- ^ The tag value for the elements to select.
+            -> VM CoreExpr
 packByTagPD ty xs len tags t
   = liftM (`mkApps` [xs, len, tags, t])
-          (paMethod packByTagPDVar "packByTagPD" ty)
-
-
--- | Combine some arrays based on a selector.
---     The selector says which source array to choose for each element of the
---     resulting array.
-combinePD 
-	:: Type		-- ^ Element type
-	-> CoreExpr	-- ^ Length of resulting array
-	-> CoreExpr	-- ^ Selector.
-	-> [CoreExpr]	-- ^ Arrays to combine.
-	-> VM CoreExpr
+          (paMethod packByTagPDVar packByTagPD_PrimVar ty)
 
+-- |Combine some arrays based on a selector.  The selector says which source array to choose for each
+-- element of the resulting array.
+--
+combinePD :: Type         -- ^ Element type
+          -> CoreExpr     -- ^ Length of resulting array
+          -> CoreExpr     -- ^ Selector.
+          -> [CoreExpr]   -- ^ Arrays to combine.
+          -> VM CoreExpr
 combinePD ty len sel xs
   = liftM (`mkApps` (len : sel : xs))
-          (paMethod (combinePDVar n) ("combine" ++ show n ++ "PD") ty)
+          (paMethod (combinePDVar n) (combinePD_PrimVar n) ty)
   where
     n = length xs
 
-
--- | Like `replicatePD` but use the lifting context in the vectoriser state.
+-- |Like `replicatePD` but use the lifting context in the vectoriser state.
+--
 liftPD :: CoreExpr -> VM CoreExpr
 liftPD x
   = do
@@ -129,6 +136,7 @@ liftPD x
 
 
 -- Scalars --------------------------------------------------------------------
+
 zipScalars :: [Type] -> Type -> VM CoreExpr
 zipScalars arg_tys res_ty
   = do
@@ -139,7 +147,6 @@ zipScalars arg_tys res_ty
     where
       ty_args = arg_tys ++ [res_ty]
 
-
 scalarClosure :: [Type] -> Type -> CoreExpr -> CoreExpr -> VM CoreExpr
 scalarClosure arg_tys res_ty scalar_fun array_fun
   = do
@@ -147,19 +154,3 @@ scalarClosure arg_tys res_ty scalar_fun array_fun
       pas <- mapM paDictOfType (init arg_tys)
       return $ Var ctr `mkTyApps` (arg_tys ++ [res_ty])
                        `mkApps`   (pas ++ [scalar_fun, array_fun])
-
-
-
-{-
-boxExpr :: Type -> VExpr -> VM VExpr
-boxExpr ty (vexpr, lexpr)
-  | Just (tycon, []) <- splitTyConApp_maybe ty
-  , isUnLiftedTyCon tycon
-  = do
-      r <- lookupBoxedTyCon tycon
-      case r of
-        Just tycon' -> let [dc] = tyConDataCons tycon'
-                       in
-                       return (mkConApp dc [vexpr], lexpr)
-        Nothing     -> return (vexpr, lexpr)
--}
