@@ -1,3 +1,4 @@
+{-# LANGUAGE UnboxedTuples #-}
 {-# OPTIONS_GHC -fno-warn-warnings-deprecations #-}
 -- The -fno-warn-warnings-deprecations flag is a temporary kludge.
 -- While working on this module you are encouraged to remove it and fix
@@ -23,18 +24,20 @@ module Language.Haskell.TH.Syntax(
 	Quasi(..), Lift(..), liftString,
 
 	Q, runQ, 
-	report,	recover, reify,
+	report,	recover, reify, 
+        lookupTypeName, lookupValueName,
 	location, runIO,
-        isClassInstance, classInstances,
+        isInstance, reifyInstances,
 
 	-- * Names
 	Name(..), mkName, newName, nameBase, nameModule,
         showName, showName', NameIs(..),
 
 	-- * The algebraic data types
+	-- $infix
 	Dec(..), Exp(..), Con(..), Type(..), TyVarBndr(..), Kind(..),Cxt,
 	Pred(..), Match(..),  Clause(..), Body(..), Guard(..), Stmt(..),
-	Range(..), Lit(..), Pat(..), FieldExp, FieldPat, ClassInstance(..),
+	Range(..), Lit(..), Pat(..), FieldExp, FieldPat, 
 	Strict(..), Foreign(..), Callconv(..), Safety(..), Pragma(..),
 	InlineSpec(..),	StrictType, VarStrictType, FunDep(..), FamFlavour(..),
 	Info(..), Loc(..), CharPos,
@@ -45,6 +48,7 @@ module Language.Haskell.TH.Syntax(
 	NameFlavour(..), NameSpace (..), 
 	mkNameG_v, mkNameG_d, mkNameG_tc, Uniq, mkNameL, mkNameU,
  	tupleTypeName, tupleDataName,
+	unboxedTupleTypeName, unboxedTupleDataName,
 	OccName, mkOccName, occString,
 	ModName, mkModName, modString,
 	PkgName, mkPkgName, pkgString
@@ -55,6 +59,7 @@ import GHC.Base		( Int(..), Int#, (<#), (==#) )
 import Language.Haskell.TH.Syntax.Internals
 import Data.Data (Data(..), Typeable, mkConstr, mkDataType, constrIndex)
 import qualified Data.Data as Data
+import Control.Applicative( Applicative(..) )
 import Data.IORef
 import System.IO.Unsafe	( unsafePerformIO )
 import Control.Monad (liftM)
@@ -67,7 +72,7 @@ import Data.Char        ( isAlpha )
 --
 -----------------------------------------------------
 
-class (Monad m, Functor m) => Quasi m where
+class (Monad m, Applicative m) => Quasi m where
   qNewName :: String -> m Name
 	-- ^ Fresh names
 
@@ -79,16 +84,21 @@ class (Monad m, Functor m) => Quasi m where
            -> m a		-- ^ Recover from the monadic 'fail'
  
 	-- Inspect the type-checker's environment
-  qReify :: Name -> m Info
-  qClassInstances :: Name -> [Type] -> m [Name]
-  		      -- Is (cls tys) an instance?
-		      -- Returns list of matching witnesses
+  qLookupName :: Bool -> String -> m (Maybe Name)
+       -- True <=> type namespace, False <=> value namespace
+  qReify          :: Name -> m Info
+  qReifyInstances :: Name -> [Type] -> m [Dec]
+       -- Is (n tys) an instance?
+       -- Returns list of matching instance Decs 
+       --    (with empty sub-Decs)
+       -- Works for classes and type functions
 
   qLocation :: m Loc
 
   qRunIO :: IO a -> m a
   -- ^ Input/output (dangerous)
 
+  qAddDependentFile :: FilePath -> m ()
 
 -----------------------------------------------------
 --	The IO instance of Quasi
@@ -109,10 +119,12 @@ instance Quasi IO where
   qReport True  msg = hPutStrLn stderr ("Template Haskell error: " ++ msg)
   qReport False msg = hPutStrLn stderr ("Template Haskell error: " ++ msg)
 
+  qLookupName _ _     = badIO "lookupName"
   qReify _            = badIO "reify"
-  qClassInstances _ _ = badIO "classInstances"
+  qReifyInstances _ _ = badIO "classInstances"
   qLocation    	      = badIO "currentLocation"
   qRecover _ _ 	      = badIO "recover" -- Maybe we could fix this?
+  qAddDependentFile _ = badIO "addDependentFile"
 
   qRunIO m = m
   
@@ -146,6 +158,10 @@ instance Monad Q where
 instance Functor Q where
   fmap f (Q x) = Q (fmap f x)
 
+instance Applicative Q where 
+  pure x = Q (pure x) 
+  Q f <*> Q x = Q (f <*> x) 
+
 ----------------------------------------------------
 -- Packaged versions for the programmer, hiding the Quasi-ness
 newName :: String -> Q Name
@@ -159,17 +175,26 @@ recover :: Q a -- ^ recover with this one
         -> Q a
 recover (Q r) (Q m) = Q (qRecover r m)
 
+-- We don't export lookupName; the Bool isn't a great API
+-- Instead we export lookupTypeName, lookupValueName
+lookupName :: Bool -> String -> Q (Maybe Name)
+lookupName ns s = Q (qLookupName ns s)
+
+lookupTypeName, lookupValueName :: String -> Q (Maybe Name)
+lookupTypeName  s = Q (qLookupName True s)
+lookupValueName s = Q (qLookupName False s)
+
 -- | 'reify' looks up information about the 'Name'
 reify :: Name -> Q Info
 reify v = Q (qReify v)
 
 -- | 'classInstances' looks up instaces of a class
-classInstances :: Name -> [Type] -> Q [Name]
-classInstances cls tys = Q (qClassInstances cls tys)
+reifyInstances :: Name -> [Type] -> Q [Dec]
+reifyInstances cls tys = Q (qReifyInstances cls tys)
 
-isClassInstance :: Name -> [Type] -> Q Bool
-isClassInstance cls tys = do { dfuns <- classInstances cls tys
-                             ; return (not (null dfuns)) }
+isInstance :: Name -> [Type] -> Q Bool
+isInstance nm tys = do { decs <- reifyInstances nm tys
+                       ; return (not (null decs)) }
 
 -- | 'location' gives you the 'Location' at which this
 -- computation is spliced.
@@ -186,14 +211,23 @@ location = Q qLocation
 runIO :: IO a -> Q a
 runIO m = Q (qRunIO m)
 
+-- | Record external files that runIO is using (dependent upon).
+-- The compiler can then recognize that it should re-compile the file using this TH when the external file changes.
+-- Note that ghc -M will still not know about these dependencies - it does not execute TH.
+-- Expects an absolute file path.
+addDependentFile :: FilePath -> Q ()
+addDependentFile fp = Q (qAddDependentFile fp)
+
 instance Quasi Q where
-  qNewName  	  = newName
-  qReport   	  = report
-  qRecover  	  = recover 
-  qReify    	  = reify
-  qClassInstances = classInstances
-  qLocation 	  = location
-  qRunIO    	  = runIO
+  qNewName  	    = newName
+  qReport   	    = report
+  qRecover  	    = recover 
+  qReify    	    = reify
+  qReifyInstances   = reifyInstances
+  qLookupName       = lookupName
+  qLocation 	    = location
+  qRunIO    	    = runIO
+  qAddDependentFile = addDependentFile
 
 
 ----------------------------------------------------
@@ -284,8 +318,8 @@ instance (Lift a, Lift b, Lift c, Lift d, Lift e, Lift f, Lift g)
 
 
 trueName, falseName :: Name
-trueName  = mkNameG DataName "ghc-prim" "GHC.Bool" "True"
-falseName = mkNameG DataName "ghc-prim" "GHC.Bool" "False"
+trueName  = mkNameG DataName "ghc-prim" "GHC.Types" "True"
+falseName = mkNameG DataName "ghc-prim" "GHC.Types" "False"
 
 nothingName, justName :: Name
 nothingName = mkNameG DataName "base" "Data.Maybe" "Nothing"
@@ -358,16 +392,12 @@ data Name = Name OccName NameFlavour deriving (Typeable, Data)
 data NameFlavour
   = NameS           -- ^ An unqualified name; dynamically bound
   | NameQ ModName   -- ^ A qualified name; dynamically bound
-
   | NameU Int#      -- ^ A unique local name
-
-
   | NameL Int#      -- ^ Local name bound outside of the TH AST
   | NameG NameSpace PkgName ModName -- ^ Global name bound outside of the TH AST:
                 -- An original name (occurrences only, not binders)
-                --
-				-- Need the namespace too to be sure which 
-				-- thing we are naming
+		-- Need the namespace too to be sure which 
+		-- thing we are naming
   deriving ( Typeable )
 
 -- |
@@ -557,24 +587,42 @@ showName' ni nm
 instance Show Name where
   show = showName
 
--- 	Tuple data and type constructors
+-- Tuple data and type constructors
 tupleDataName :: Int -> Name    -- ^ Data constructor
 tupleTypeName :: Int -> Name    -- ^ Type constructor
 
-tupleDataName 0 = mk_tup_name 0 DataName 
+tupleDataName 0 = mk_tup_name 0 DataName
 tupleDataName 1 = error "tupleDataName 1"
-tupleDataName n = mk_tup_name (n-1) DataName 
+tupleDataName n = mk_tup_name (n-1) DataName
 
-tupleTypeName 0 = mk_tup_name 0 TcClsName 
+tupleTypeName 0 = mk_tup_name 0 TcClsName
 tupleTypeName 1 = error "tupleTypeName 1"
-tupleTypeName n = mk_tup_name (n-1) TcClsName 
+tupleTypeName n = mk_tup_name (n-1) TcClsName
 
 mk_tup_name :: Int -> NameSpace -> Name
 mk_tup_name n_commas space
   = Name occ (NameG space (mkPkgName "ghc-prim") tup_mod)
   where
     occ = mkOccName ('(' : replicate n_commas ',' ++ ")")
-    -- XXX Should it be GHC.Unit for 0 commas?
+    tup_mod = mkModName "GHC.Tuple"
+
+-- Unboxed tuple data and type constructors
+unboxedTupleDataName :: Int -> Name    -- ^ Data constructor
+unboxedTupleTypeName :: Int -> Name    -- ^ Type constructor
+
+unboxedTupleDataName 0 = error "unboxedTupleDataName 0"
+unboxedTupleDataName 1 = error "unboxedTupleDataName 1"
+unboxedTupleDataName n = mk_unboxed_tup_name (n-1) DataName
+
+unboxedTupleTypeName 0 = error "unboxedTupleTypeName 0"
+unboxedTupleTypeName 1 = error "unboxedTupleTypeName 1"
+unboxedTupleTypeName n = mk_unboxed_tup_name (n-1) TcClsName
+
+mk_unboxed_tup_name :: Int -> NameSpace -> Name
+mk_unboxed_tup_name n_commas space
+  = Name occ (NameG space (mkPkgName "ghc-prim") tup_mod)
+  where
+    occ = mkOccName ("(#" ++ replicate n_commas ',' ++ "#)")
     tup_mod = mkModName "GHC.Tuple"
 
 
@@ -605,7 +653,7 @@ data Info
     --   and a list of its instances
     ClassI 
         Dec             -- Declaration of the class
-        [ClassInstance]	-- The instances of that class
+        [InstanceDec]	-- The instances of that class
 
   | ClassOpI
 	Name	-- The class op itself
@@ -613,7 +661,12 @@ data Info
 	Name 	-- Name of the parent class
 	Fixity
 
-  | TyConI Dec
+  | TyConI 
+        Dec
+
+  | FamilyI	-- Type/data families
+        Dec
+        [InstanceDec]
 
   | PrimTyConI 	-- Ones that can't be expressed with a data type 
 		-- decl, such as (->), Int#
@@ -640,15 +693,12 @@ data Info
 	Type	-- What it is bound to
   deriving( Show, Data, Typeable )
 
--- | 'ClassInstance' desribes a single instance of a class
-data ClassInstance 
-  = ClassInstance {
-      ci_dfun :: Name,	  -- The witness
-      ci_tvs  :: [TyVarBndr], 
-      ci_cxt  :: Cxt,
-      ci_cls  :: Name,  
-      ci_tys  :: [Type]
-    } deriving( Show, Data, Typeable )
+-- | 'InstanceDec' desribes a single instance of a class or type function
+-- It is just a 'Dec', but guaranteed to be one of the following:
+--   InstanceD (with empty [Dec])
+--   DataInstD or NewtypeInstD (with empty derived [Name])
+--   TySynInstD
+type InstanceDec = Dec
 
 data Fixity          = Fixity Int FixityDirection
     deriving( Eq, Show, Data, Typeable )
@@ -668,6 +718,68 @@ defaultFixity = Fixity maxPrecedence InfixL
 --
 -----------------------------------------------------
 
+{- $infix #infix#
+Note [Unresolved infix]
+~~~~~~~~~~~~~~~~~~~~~~~
+
+When implementing antiquotation for quasiquoters, one often wants
+to parse strings into expressions:
+
+> parse :: String -> Maybe 'Exp'
+
+But how should we parse @a + b * c@? If we don't know the fixities of
+@+@ and @*@, we don't know whether to parse it as @a + (b * c)@ or @(a
++ b) * c@.
+
+In cases like this, use 'UInfixE' or 'UInfixP', which stand for
+\"unresolved infix expression\" and \"unresolved infix pattern\". When
+the compiler is given a splice containing a tree of @UInfixE@
+applications such as
+
+> UInfixE
+>   (UInfixE e1 op1 e2)
+>   op2
+>   (UInfixE e3 op3 e4)
+
+it will look up and the fixities of the relevant operators and
+reassociate the tree as necessary.
+
+  * trees will not be reassociated across 'ParensE' or 'ParensP',
+    which are of use for parsing expressions like
+
+    > (a + b * c) + d * e
+
+  * 'InfixE' and 'InfixP' expressions are never reassociated.
+
+  * The 'UInfixE' constructor doesn't support sections. Sections
+    such as @(a *)@ have no ambiguity, so 'InfixE' suffices. For longer
+    sections such as @(a + b * c -)@, use an 'InfixE' constructor for the
+    outer-most section, and use 'UInfixE' constructors for all
+    other operators:
+
+    > InfixE
+    >   Just (UInfixE ...a + b * c...)
+    >   op
+    >   Nothing
+
+    Sections such as @(a + b +)@ and @((a + b) +)@ should be rendered
+    into 'Exp's differently:
+
+    > (+ a + b)   ---> InfixE Nothing + (Just $ UInfixE a + b)
+    >                    -- will result in a fixity error if (+) is left-infix
+    > (+ (a + b)) ---> InfixE Nothing + (Just $ ParensE $ UInfixE a + b)
+    >                    -- no fixity errors
+
+  * Quoted expressions such as
+
+    > [| a * b + c |] :: Q Exp
+    > [p| a : b : c |] :: Q Pat
+
+    will never contain 'UInfixE', 'UInfixP', 'ParensE', or 'ParensP'
+    constructors.
+
+-}
+
 data Lit = CharL Char 
          | StringL String 
          | IntegerL Integer     -- ^ Used for overloaded and non-overloaded
@@ -691,8 +803,15 @@ data Pat
   = LitP Lit                      -- ^ @{ 5 or 'c' }@
   | VarP Name                     -- ^ @{ x }@
   | TupP [Pat]                    -- ^ @{ (p1,p2) }@
+  | UnboxedTupP [Pat]             -- ^ @{ (# p1,p2 #) }@
   | ConP Name [Pat]               -- ^ @data T1 = C1 t1 t2; {C1 p1 p1} = e@
   | InfixP Pat Name Pat           -- ^ @foo ({x :+ y}) = e@
+  | UInfixP Pat Name Pat          -- ^ @foo ({x :+ y}) = e@
+                                  --
+                                  -- See Note [Unresolved infix] at "Language.Haskell.TH.Syntax#infix"
+  | ParensP Pat                   -- ^ @{(p)}@
+                                  --
+                                  -- See Note [Unresolved infix] at "Language.Haskell.TH.Syntax#infix"
   | TildeP Pat                    -- ^ @{ ~p }@
   | BangP Pat                     -- ^ @{ !p }@
   | AsP Name Pat                  -- ^ @{ x \@ p }@
@@ -734,8 +853,15 @@ data Exp
     -- Maybe there should be a var-or-con type?
     -- Or maybe we should leave it to the String itself?
 
+  | UInfixE Exp Exp Exp                -- ^ @{x + y}@
+                                       --
+                                       -- See Note [Unresolved infix] at "Language.Haskell.TH.Syntax#infix"
+  | ParensE Exp                        -- ^ @{ (e) }@
+                                       --
+                                       -- See Note [Unresolved infix] at "Language.Haskell.TH.Syntax#infix"
   | LamE [Pat] Exp                     -- ^ @{ \ p1 p2 -> e }@
   | TupE [Exp]                         -- ^ @{ (e1,e2) }  @
+  | UnboxedTupE [Exp]                  -- ^ @{ (# e1,e2 #) }  @
   | CondE Exp Exp Exp                  -- ^ @{ if e1 then e2 else e3 }@
   | LetE [Dec] Exp                     -- ^ @{ let x=e1;   y=e2 in e3 }@
   | CaseE Exp [Match]                  -- ^ @{ case e of m1; m2 }@
@@ -820,7 +946,7 @@ data Foreign = ImportF Callconv Safety String Name Type
 data Callconv = CCall | StdCall
           deriving( Show, Eq, Data, Typeable )
 
-data Safety = Unsafe | Safe | Threadsafe
+data Safety = Unsafe | Safe | Interruptible
         deriving( Show, Eq, Data, Typeable )
 
 data Pragma = InlineP     Name InlineSpec
@@ -839,7 +965,7 @@ data Pred = ClassP Name [Type]    -- ^ @Eq (Int, a)@
           | EqualP Type Type      -- ^ @F a ~ Bool@
           deriving( Show, Eq, Data, Typeable )
 
-data Strict = IsStrict | NotStrict
+data Strict = IsStrict | NotStrict | Unpacked
          deriving( Show, Eq, Data, Typeable )
 
 data Con = NormalC Name [StrictType]          -- ^ @C Int a@
@@ -855,6 +981,7 @@ data Type = ForallT [TyVarBndr] Cxt Type  -- ^ @forall <vars>. <ctxt> -> <type>@
           | VarT Name                     -- ^ @a@
           | ConT Name                     -- ^ @T@
           | TupleT Int                    -- ^ @(,), (,,), etc.@
+          | UnboxedTupleT Int             -- ^ @(#,#), (#,,#), etc.@
           | ArrowT                        -- ^ @->@
           | ListT                         -- ^ @[]@
           | AppT Type Type                -- ^ @T a b@
